function [f1,f2,f3,f4] = CollinearNodeForce(x1,x2,x3,x4,bp,b,a,C)
%compute the nodal forces for segment 1-2 and 3-4
%segments alligned in the order 1-2-3-4, along 1-2 direction
%excluding the case 1-2 and 3-4 have overlappings

%inputs:
%x1,x2,bp (1x3) endpoints of first dislocation segment starting at x1 ending at x2 with burgers vector bp
%x3,x4,b (1x3) endpoints of first dislocation segment starting at x3 ending at x4 with burgers vector b
%a: core cut-off radius
%C: elastic constant tensor

if(~exist('a'))
    a = 1e-3;%default core radius
end

%line direction of x1-x2
xi = (x2-x1)/norm(x2-x1);

%stress factors for 1-2, 3-4
fac_co_12 = fac_co(xi,bp,C,'m',a);
fac_co_34 = fac_co(xi,b,C,'m',a);

%force factors 
G1 = cross((fac_co_34*bp')',xi);G2 = G1;
G3 = cross((fac_co_12*b')',xi);G4 = G3;

%lengths needed
L1 = norm(x2-x1);
L2 = norm(x4-x3);
L3 = norm(x3-x2);

%calculate scalar coefficient P
if L3 >= a
    P1 = (-L3*log(L3)+L3*log(L1+L3)+(L2+L3)*log((L2+L3)/(L1+L2+L3)))/L1;
    P2 = ((L1+L3)*log(L3/(L1+L3))-(L1+L2+L3)*log((L2+L3)/(L1+L2+L3)))/L1;
    P3 = log((L1+L3)*(L2+L3)/L3/(L1+L2+L3))-(-L1*log(L1+L3)+L3*log(L3/(L1+L3)/(L2+L3))+(L1+L3)*log(L1+L2+L3))/L2;  
    P4 = (-L1*log(L1+L3)+L3*log(L3/(L1+L3)/(L2+L3))+(L1+L3)*log(L1+L2+L3))/L2;
else
    P1 = 1 - a/2/L1+L3/L1-L3^2/2/a/L1 - (-a+L1+L3+L3*log(a/(L1+L3)))/L1 + (L2+L3)*log((L2+L3)/(L1+L2+L3))/L1;
    P2 = -2 + a/2/L1+L3/a-L3/L1+L3^2/2/a/L1+(-a+L1+L3+(L1+L3)*log(a/(L1+L3)))/L1+(L1+L2+L3)*log(1+L1/(L2+L3))/L1;
    P3 =  2 - a/2/L2 - L3/a + L3/L2 - L3^2/2/a/L2 + (a-L2-L3+(L2+L3)*log((L2+L3)/a))/L2 + (L1+L3)*log((L1+L3)/(L1+L2+L3))/L2 - log((L1+L2+L3)/(L1+L3));  
    P4 = (a-L3)^2/2/a/L2 + (-a+L2+L3+L3*log(a)-L3*log(L2+L3))/L2 - (L2+(L1+L3)*log((L1+L3)/(L1+L2+L3)))/L2;    
end

%collinear nodal forces
f1 = P1*G1;f2 = P2*G2;f3 = P3*G3;f4 = P4*G4;