function [f1,f2,f3,f4]=RemoteNodeForce_aniso_quad(x1,x2,x3,x4,bp,b,a,C,tol,flag_rem,ntrans)
% calculates the forces between dislocation nodes by numerical integration
%Adaptive simpson method was used

%inputs:
%x1,x2,bp (1x3) :first dislocation segment starting at x1 ending at x2 with burgers vector bp
%x3,x4,b (1x3) :first dislocation segment starting at x3 ending at x4 with burgers vector b
%a: core cut-off radius
%C: elastic constant tensor
%tol: absolute error tolerance (used for integration)
%flag_rem: flag of nodes (7 means fixed, 0 means free)
%ntrans: exponent of coordinate transformation(default value is 1)
%
% outputs: f1,f2,f3,f4 is the force on nodes located at x1, x2, x3, x4 respectively
%

% default tolerance
if(~exist('rtol'))
    tol = 1e-4;
end

%default flag, 0(free)
if(~exist('flag_rem'))
    flag_rem = [0 0 0 0];
end

%default ntrans, 1(no transformation)
if(~exist('ntrans'))
    ntrans = 1;
end

% special cases 
%
% Four nodes are all fixed
if(min(flag_rem == 7))
    f1=zeros(1,3);
    f2=f1;f3=f1;f4=f1;
    return;
end
%
% collinear case
t1 = (x2-x1)/norm(x2-x1);
col_tol = 1e-6;
d3 = norm((x3-x2)-dot((x3-x2),t1)*t1);
d4 = norm((x4-x2)-dot((x4-x2),t1)*t1);
if max(d3,d4)<col_tol
    %collinear case
    if dot((x3-x2),t1) <= dot((x4-x2),t1)
        [f1,f2,f3,f4] = CollinearNodeForce(x1,x2,x3,x4,bp,b,a,C);
        return;
    else
        [f1,f2,f4,f3] = CollinearNodeForce(x1,x2,x4,x3,bp,b,a,C);
        return;
    end
end
%
% general cases
if( (flag_rem(1) == 7) && (flag_rem(2) == 7) )
    f1=zeros(1,3);
    f2=zeros(1,3);
else
    [f1,f2]=RemoteNodeForce_aniso_int(x1,x2,x3,x4,bp,b,a,C,tol,flag_rem([1,2]),ntrans);
end
%    
% general cases
if( (flag_rem(3) == 7) && (flag_rem(4) == 7) )
    f3=zeros(1,3);
    f4=zeros(1,3);
else
    [f4,f3]=RemoteNodeForce_aniso_int(x4,x3,x2,x1,-b,-bp,a,C,tol,flag_rem([4,3]),ntrans);
end

return;


%================================================================
%    
function [f1,f2]=RemoteNodeForce_aniso_int(x1,x2,x3,x4,bp,b,a,C,tol,flag_rem,ntrans);
% calculate the remote nodal force as a function of integration numbers
% integration points are always evenly distribution on segment 1-2
% depending on number of integration points N
% n: exponent of coordinate transformation
%Adaptive simpson method was used

if(~exist('flag_rem'))
    flag_rem = [0 0];%free
end

if(~exist('ntrans'))
    ntrans = 1;%no coordinate transformation
end

%
flag1 = flag_rem(1);flag2 = flag_rem(2);

%initializing f's
f1=zeros(1,3); f2 = f1;

%    
if ( (flag1 == 7) && (flag2 == 7) )
    return
end

%parameters needed for preparing integrand (8x3 matrices)
param = [x1; x2; x3; x4; bp; b; [a, flag_rem(1), flag_rem(2)]; [ntrans 0 0]];

%numerical integration
[Q,neval] = quadv('node_force_integrand',0,1,tol,[],param,C);

f1 = Q(1:3); f2 = Q(4:6);

return;

nt_f = node_force_integrand(s, param)