function SIGMA = SIG(t,m,b,C,meth,Nint)
%calculate the stress angular factor SIGMA

%t: (1x3) unit row vector, indicating dislocation orientation
%m:(1x3) unit vector perpendicular to t, used to define theta
%b:(1x3) Burgers vector
%C: (3x3x3x3, rank-4 tensor) elastic constant tensor
%meth: matrix or integral method to compute QBS matrices

%meth must be specified

t=t/norm(t);n=cross(t,m);n=n/norm(n);

if meth == 'm'    
    [Q_p,B_p,S_p] = stroh_p_m(t,n,b,C);
    [Q,B,S]=stroh_m(t,C);
elseif exist('Nint')
    [Q_p,B_p,S_p] = stroh_p_i(t,n,b,C,Nint);
    [Q,B,S]=stroh_i(t,C,Nint);
else
    [Q_p,B_p,S_p] = stroh_p_i(t,n,b,C);
    [Q,B,S]=stroh_i(t,C);
end

%Prepare (mn) matrices and intermediate matrix Ma0
z3 = zeros(3); nn=z3; nm=z3;
for j=1:3
     for k=1:3
         for i=1:3
             for l=1:3
                 nn(j,k)=nn(j,k)+n(i)*C(i,j,k,l)*n(l);
                 nm(j,k)=nm(j,k)+n(i)*C(i,j,k,l)*m(l);
             end
         end
     end
end

%Define nn_1
nn_1=inv(nn);

%Intermediate matrix Ma0
Ma0=nn_1*(B+nm*S);

%Calculate the stress angular factor
SIGMA=z3; 

for i=1:3
    for j=1:3
        for k=1:3
            for l=1:3
                for s=1:3
                    SIGMA(i,j)=SIGMA(i,j)+C(i,j,k,l)*b(s)*(-m(l)*S(k,s)+n(l)*Ma0(k,s))/2/pi;
                end
            end
        end
    end
end