function SIGMA_p = SIG_p(t,n,b,C,meth,Nint)
%calculate the derivative of angular stress factor, SIGMA, wrt theta

%t: (1x3) unit row vector, indicating dislocation orientation
%n:(1x3) unit vector perpendicular to t, used to define theta
%b:(1x3) Burgers vector
%C: (3x3x3x3, rank-4 tensor) elastic constant tensor
%meth: method used to compute QBS and their angular derivatives

%method must be specified

t=t/norm(t); n=n/norm(n);
m=cross(n,t);

if meth == 'm'
    [Q_p,B_p,S_p] = stroh_p_m(t,n,b,C);
    [Q,B,S]=stroh_m(t, C);
elseif exist('Nint')
    [Q_p,B_p,S_p] = stroh_p_i(t,n,b,C,Nint);
    [Q,B,S]=stroh_i(t, C,Nint);
else
    [Q_p,B_p,S_p] = stroh_p_i(t,n,b,C);
    [Q,B,S]=stroh_i(t, C);
end

format long

z3 = zeros(3); nn=z3; nt=z3; nm=z3;

%defining ( )'s
for i=1:3
    for j=1:3
        for k=1:3
            for l=1:3
                nn(j,k)=nn(j,k)+n(i)*C(i,j,k,l)*n(l);
                nt(j,k)=nt(j,k)+n(i)*C(i,j,k,l)*t(l);
                nm(j,k)=nm(j,k)+n(i)*C(i,j,k,l)*m(l);
            end
        end
    end
end
nn_1=inv(nn);

%intermediate matrix
Ma = zeros(3);
Ma = nn_1*(B_p+nm*S_p-nt*S);

SIGMA_p=z3;

%calculate SIGMA_p
for j=1:3
    for k=1:3
        for i=1:3
            for s=1:3
                for p=1:3
                    SIGMA_p(j,k)=SIGMA_p(j,k)+C(j,k,i,p)*b(s)*...
                        (t(p)*S(i,s)-m(p)*S_p(i,s)+n(p)*Ma(i,s))/2/pi;
                end
            end
        end
    end
end