function C = cubicelasticconst(c11,c12,c44,meth,pert)
%generate elastic stiffness tensor C_ijkl
%C_ijkl is a rank 4 tensor
%pert: perturbation to c44 when the medium is isotropic

%==============================================
%perturbation for isotropic medium
tol = 1e-6; %tolerance

if nargin < 4
    meth = 'm'; pert = 1e-6; % default perturbation rate if the media is isotropic
elseif nargin == 5
    if meth == 'm'
        pert = 1e-6;
    elseif meth ~= 'i'%only pert is given
        pert = meth;
        meth = 'm';
    end
end

%perturb c44 specifically for matrix formalism
if meth == 'm' && abs(2*c44+c12-c11) < tol
    c44 = c44 + pert;
end

%==============================================

%construct C_ijkl

%initializing C
C=zeros(3,3,3,3);

%construct delta function
delta=eye(3);

%Define H
H=2*c44+c12-c11;

%For cubic crystals
for i=1:3
    for j=1:3
        for k=1:3
            for l=1:3
                C(i,j,k,l)=c44*(delta(i,k)*delta(j,l)+delta(i,l)*delta(j,k))+...
                    c12*delta(i,j)*delta(k,l)-H*delta(i,j)*delta(k,l)*delta(i,k);
            end
        end
    end
end