%Dislocation Dynamics simulation in 3-dimension
%
%the best way to run it is by "rundd3d"
%
%Features:
%infinite boundary condition (no pbc)
%linear mobility law (mobfcc0,mobfcc1)
%no remesh, no collision detection, no reaction
%N^2 interaction (no neighbor list, no fast multipole)

%Data structure:
%NMAX:    maximum number of nodes (including disabled ones)
%LINKMAX: maximum number of links (including disabled ones)
%rn: (NMAX,4) array of nodal positions (last column is flag: -1 means disabled)
%vn: (NMAX,3) array of nodal velocities
%fn: (NMAX,3) array of nodal forces
%links: (LINKMAX,8) array of links (id1,id2,bx,by,bz,nx,ny,nz)


% set default remesh rule as remesh.m
%if(~exist('remeshrule'))
    %remeshrule = 'remesh';
%end

% set default plot method as plotnodes.m
if(~exist('plotmethod'))
    plotmethod = 'plotnodes';
end

%default value if run by itself (e.g. not through "rundd3d")
% cleanup the empty node and link entries at the end of the initial data structures
[rn,links]=cleanupnodes(rn,links);

% genererate the connectivity list from the list of links
[connectivity,linksinconnect]=genconnectivity(rn,links,maxconnections);
consistencycheck(rn,links,connectivity,linksinconnect,elasticinteraction);

%plot dislocation structure
figure(1);
feval(plotmethod,rn,links,plim); view(viewangle); xlim([-plim plim]); ylim([-plim plim]); zlim([-plim plim]);
drawnow

data=zeros(totalsteps,1);
if(~exist('dt'))
    dt=dt0;
end
dt=min(dt,dt0);
mdold=10;
counter=0;
for curstep=1:totalsteps
    %integrating equation of motion
    [rnnew,vn,dt,fn,fseg]=feval(integrator,rn,dt,dt0,C,a,links,connectivity,appliedstress,rmax,rntol,mobility,elasticinteraction,Ec);
    % plastic strain and plastic spin calculations
    [ep_inc,wp_inc]=calcplasticstrainincrement(rnnew,rn,links,(2*plim)^3);
    
%% Disable velocity printi    
    if(mod(curstep,printfreq)==0)
        disp(sprintf('step%3d dt=%e v%d=(%e,%e,%e)',...
            curstep,dt,printnode,vn(printnode,1),vn(printnode,2),vn(printnode,3)));
    end
    
%     if(mod(curstep,printfreq)==0)
%         disp(sprintf('step%3d dt=%e',curstep,dt));
%     end
    if(mod(curstep,plotfreq)==0)
        feval(plotmethod,rn,links,plim,vn);  xlim([-plim plim]); ylim([-plim plim]); zlim([-plim plim]);
        view(viewangle);
        drawnow
        pause(0.01);
    end
    
    rnnew=[rnnew(:,1:3) vn rnnew(:,4)];
    linksnew=links;
    connectivitynew=connectivity;
    linksinconnectnew=linksinconnect;
    fsegnew=fseg;
    if(doseparation)
        %spliting of nodes with 4 or more connections
        [rnnew,linksnew,connectivitynew,linksinconnectnew,fsegnew]=separation(rnnew,linksnew,connectivitynew,linksinconnectnew,fsegnew,mobility,C,a,2*rann,appliedstress,elasticinteraction);
    
    end
    if(docollision)
        %collision detection and handling
        [rnnew,linksnew,connectivitynew,linksinconnectnew,fsegnew]=collision(rnnew,linksnew,connectivitynew,linksinconnectnew,fsegnew,rann,C,a,mobility,appliedstress,elasticinteraction);
    end

   %if(~exist('remeshrule'))
        %remesh
        if(doremesh)
            remeshrule = 'remesh';            
            [rnnew,linksnew,connectivitynew,linksinconnectnew,fsegnew]=feval(remeshrule,rnnew,linksnew,connectivitynew,linksinconnectnew,fsegnew,lmin,lmax,areamin,areamax,C,a,appliedstress,mobility,elasticinteraction,Ec);      
        end
    %else
       % [rnnew,linksnew,connectivitynew,linksinconnectnew,fsegnew]=feval(remeshrule,rnnew,linksnew,connectivitynew,linksinconnectnew,fsegnew,lmin,lmax,areamin,areamax,C,a,appliedstress,mobility,elasticinteraction);      
    %end
   
    rn=[rnnew(:,1:3) rnnew(:,7)];
    vn=rnnew(:,4:6);
    links=linksnew;
    connectivity=connectivitynew;
    linksinconnect=linksinconnectnew;
    fseg=fsegnew;
    %store run time information
    %time step
    data(curstep,1)=dt;
    %save restart
end
%save restart

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%