function int_f12 = node_force_integrand(s, param,C)
%define the integrand needed for calculating Peach-Koehler force
%on segment 1-2 due to the stress field of segment 3-4

%s is the variable for integration, varing from 0 to 1
%param = [x1; x2; x3; x4; bp; b; [a, flag_rem(1), flag_rem(2)]; [ntrans 0 0]];
%C: (3x3x3x3, rank-4 tensor) elastic constant tensor

x1 = param(1,:);x2 = param(2,:);
x3 = param(3,:);x4 = param(4,:);
bp = param(5,:);b  = param(6,:);
a  = param(7,1); flag1 = param(7,2); flag2 = param(7,3);
n  = param(8,1);

%transformed coordinate
alpha = s^n;
dalphads = n*s^(n-1);

%field point P on segment 1-2
seg_12_x = x1(1) + (x2(1)-x1(1)) * alpha;
seg_12_y = x1(2) + (x2(2)-x1(2)) * alpha;
seg_12_z = x1(3) + (x2(3)-x1(3)) * alpha;

P=[seg_12_x, seg_12_y, seg_12_z];

%segment direction
xi_12 = x2 - x1;

% stress field of segment 3-4 at point P
sig_34=stress_seg_WS(x3,x4,P,b,C,'m',a);

%PK force
fpk = cross((sig_34*bp')',xi_12);

% update PK force (times weighting function) only if node is free
if(flag1 == 7)
    int_f1 = [0 0 0];
else
    int_f1 = fpk*(1-alpha)*dalphads;
end

if(flag2 == 7)
    int_f2 = [0 0 0];
else
    int_f2 = fpk*alpha*dalphads;
end

int_f12 = [int_f1, int_f2];