function sigma_co = stress_co(A,B,P,b,C,meth,a,Nint)
%calculate the stress due to dislocation segment AB at P
%A,B,P are collinear
%Based on Willis-Steeds-Lothe formula

%A,B: (1x3) row vector, the starting and ending point of a segment
%P: (1x3) field point
%b:(1x3) Burgers vector
%C: (3x3x3x3, rank-4 tensor) elastic constant tensor
%meth: method used to compute QBS, 'm' or 'i'
%a: dislocation core radius

t = (B-A)/norm(B-A);

%meth and a must be specified
if (~exist('Nint'))
    fac_co_stress = fac_co(t,b,C,meth,a);
else
    fac_co_stress = fac_co(t,b,C,'i',a,Nint);
end

r1 = max(norm(P-A),a);
r2 = max(norm(P-B),a);
sigma_co =  fac_co_stress*(1/r2-1/r1);