function sigma_B = stress_inf_B(x, t, b, C, meth,Nint)
%calculate the stress field due to an inifite dislocation
%using Brown's formula

%x: (1x3) field point x
%t: (1x3) unit row vector, indicating dislocation orientation
%b: (1x3) Burgers vector
%C: (3x3x3x3, rank-4 tensor) elastic constant tensor
%meth: method used to compute QBS
%Nint is the number of integration points, for integral formalism only

if nargin == 4
    [Q,B,S]=stroh_m(t, C);
elseif nargin == 5
    [Q,B,S]=stroh(t, C,meth);
elseif nargin == 6
    [Q,B,S]=stroh_i(t, C, Nint);
end

t=t/norm(t);
%Define m and n
m = x - dot(x,t)*t; d = norm(m); 
m = m/d;
n = cross(t,m);

%define ( )'s
z3 = zeros(3);
nn=z3; nm=z3;
for j=1:3
     for k=1:3
         for i=1:3
             for l=1:3
                 nn(j,k)=nn(j,k)+n(i)*C(i,j,k,l)*n(l);
                 nm(j,k)=nm(j,k)+n(i)*C(i,j,k,l)*m(l);
             end
         end
     end
end

%define nn_1
nn_1=inv(nn);

%defining intermediate matrix Ma0
Ma0=nn_1*(B+nm*S);

%Calculate Sigma stress angular factor
SIGMA=z3;
for i=1:3
    for j=1:3
        for k=1:3
            for l=1:3
                for s=1:3
                    SIGMA(i,j)=SIGMA(i,j)+C(i,j,k,l)*b(s)*(-m(l)*S(k,s)+n(l)*Ma0(k,s))/2/pi;
                end
            end
        end
    end
end

%Calculate Stress field
sigma_B = SIGMA/d;
