function sigma_WS=stress_inf_WS(x,t,b,C,meth,Nint)
%calculate the stress field of an infinite dislocation line
%using Willis-Steeds-Lothe formula

%x: (1x3) field point x
%t: (1x3) unit row vector, indicating dislocation orientation
%b: (1x3) Burgers vector
%C: (3x3x3x3, rank-4 tensor) elastic constant tensor
%meth: method used to compute QBS
%Nint is the number of integration points, for integral formalism only

if nargin == 4
    [Q,B,S]=stroh_m(t, C);
elseif nargin == 5
    [Q,B,S]=stroh(t, C,meth);
elseif nargin == 6
    [Q,B,S]=stroh_i(t, C, Nint);
end

%redefine x
t=t/norm(t); x=x-dot(x,t)*t;
%define m and n
m=x/norm(x);
n=cross(t,m); n=n/norm(n);

%define nn,nm
%initializing nn nm mn mm
z3 = zeros(3);
nn=z3; nm=z3; 
for j=1:3
     for k=1:3
         for i=1:3
             for l=1:3
                 nn(j,k)=nn(j,k)+n(i)*C(i,j,k,l)*n(l);
                 nm(j,k)=nm(j,k)+n(i)*C(i,j,k,l)*m(l);
             end
         end
     end
end

%define alternating tensor
%initializing
epsilon=zeros(3,3,3);
epsilon(1,2,3)=1;
epsilon(2,3,1)=1;
epsilon(3,1,2)=1;
epsilon(1,3,2)=-1;
epsilon(2,1,3)=-1;
epsilon(3,2,1)=-1;

%define nn_1
nn_1=inv(nn);

d=norm(x);
%Intermediate matrices
Ma1=nn_1*nm*Q;
Ma2=nn_1*S';

sigma_WS=z3;
for i=1:3
    for j=1:3
        for k=1:3
            for l=1:3
                for s=1:3
                    for u=1:3
                        for p=1:3
                            for q=1:3
                                for w=1:3
                                    sigma_WS(i,j)=sigma_WS(i,j)+C(i,j,k,l)*epsilon(s,l,u)*b(p)*C(p,s,q,w)...
                                        *t(u)*(-m(w)*Q(k,q)+n(w)*Ma1(k,q)+n(w)*Ma2(k,q))/2/pi/d;
                                end
                            end
                        end
                    end
                end
            end
        end
    end
end