function sigma_ISO=stress_inf_iso(x,t,b,mu,nu)
%calculate the stress field due to an inifite dislocation line
%in isotropic media
%using analytical formulas

%x:(1x3) field point x
%t: (1x3) unit row vector, indicating dislocation orientation
%b:(1x3) Burgers vector
%mu: shear modulus
%nu: Poisson ratio


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


% Step 1 Vector preparation

%normalizing t
%t is not necessarily [0 0 1]
t=t/norm(t);
%redefine x, for infinite straight dislocation
%norm x is the distance from x to t
x=x-dot(x,t)*t;
d=norm(x);
%decompose b into 2 categories
%screw dislocation,scalar
bs=dot(b,t);
%edge dislocation,vector
be=b-bs*t;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Step2 Applying formulas

%initializing
sigma_ISO=zeros(3);

if norm(be)~=0
    %construct coordinate system using be(m) and t(x3)
    q1=be/norm(be);
    q3=t;
    q2=cross(q3,q1);
    %normalizing q2(n)
    q2=q2/norm(q2);
    %calculate edge dislocation first
    sigma_e=zeros(3);
    x1=dot(x,q1);
    x2=dot(x,q2);
    sigma_e(1,1)=-mu*norm(be)/2/pi/(1-nu)*x2*(3*x1^2+x2^2)/(x1^2+x2^2)^2;
    sigma_e(2,2)=mu*norm(be)/2/pi/(1-nu)*x2*(x1^2-x2^2)/(x1^2+x2^2)^2;
    sigma_e(1,2)=mu*norm(be)/2/pi/(1-nu)*x1*(x1^2-x2^2)/(x1^2+x2^2)^2;
    sigma_e(2,1)=sigma_e(1,2);
    sigma_e(3,3)=-mu*norm(be)*nu/pi/(1-nu)*x2/(x1^2+x2^2);
    %calculate screw dislocation now
    sigma_s=zeros(3);
    sigma_s(1,3)=-mu*bs/2/pi*x2/(x1^2+x2^2);
    sigma_s(3,1)=sigma_s(1,3);
    sigma_s(2,3)=mu*bs/2/pi*x1/(x1^2+x2^2);
    sigma_s(3,2)=sigma_s(2,3);
    %matrix transform
    q=[q1;q2;q3]';
    E=eye(3);
    for j=1:3
        for k=1:3
        T(j,k)=dot(q(:,j),E(:,k));
        %constructing transformation matrix T
        end
    end
    %A'=T*A*T'
    %inv(T)=T';
    %A=T'*A'*T;
    sigma_q=sigma_e+sigma_s;
    sigma_ISO=T'*sigma_q*T;
else
    %be=0
    %only screw dislocation
    %construct coordinate system using x and t
    q1=x/d;
    q3=t;
    q2=cross(q3,q1);
    %normalizing q2(n)
    q2=q2/norm(q2);
    x1=d;
    x2=0;
    sigma_s=zeros(3);
    sigma_s(1,3)=-mu*bs/2/pi*x2/(x1^2+x2^2);
    sigma_s(3,1)=sigma_s(1,3);
    sigma_s(2,3)=mu*bs/2/pi*x1/(x1^2+x2^2);
    sigma_s(3,2)=sigma_s(2,3);
    %matrix transform
    q=[q1;q2;q3]';
    E=eye(3);
    for j=1:3
        for k=1:3
        T(j,k)=dot(q(:,j),E(:,k));
        %constructing transformation matrix T
        end
    end
    %A=T'*A'*T;
    sigma_ISO=T'*sigma_s*T;
end