function sigma_seg_B=stress_seg_B(A,B,P,b,C,meth,a,Nint)
%compute the stress field of a dislocation segment AB at P
%using Brown's formula

%A,B: (1x3) row vector, the starting and ending point of a segment
%P: (1x3) field point
%b:(1x3) Burgers vector
%C: (3x3x3x3, rank-4 tensor) elastic constant tensor
%meth: method used to compute QBS, 'm' or 'i'
%a: dislocation core radius; default value for Brown's formula is zero

%meth and a must be specified
%Nint is only valid when meth == 'i'

%Step 1 Determine if it is collinear case

eps = 1e-6; 

xi=(B-A)/norm(B-A); %segment orientation
x = (P-A) - dot((P-A),xi)*xi; 
h = norm(x);

if (h < eps)
    %collinear case
    sigma_seg_B = zeros(3);%Brown's formula
    return
end

%Step 2 Compute ABP plane normal n
ev = x/h;
n = cross(xi,ev);

%Step3 Calculate angles 

tau1=(P-A)/norm(P-A);
tau2=(P-B)/norm(P-B);
m1=cross(n,tau1);
m2=cross(n,tau2);
beta1 = acos(dot(tau1,xi));
beta2 = acos(dot(tau2,xi));
 
%Step4 Calculate Angular Factors and SIGMA_prime's

if(~exist('Nint'))
    SIGMA1 = SIG(tau1,m1,b,C,meth);
    SIGMA2 = SIG(tau2,m2,b,C,meth);
    SIGMA_p1 = SIG_p(tau1,n,b,C,meth);
    SIGMA_p2 = SIG_p(tau2,n,b,C,meth);
else
    SIGMA1 = SIG(tau1,m1,b,C,'i',Nint);
    SIGMA2 = SIG(tau2,m2,b,C,'i',Nint);
    SIGMA_p1 = SIG_p(tau1,n,b,C,'i',Nint);
    SIGMA_p2 = SIG_p(tau2,n,b,C,'i',Nint);
end
 
%Step4 Calculate stress field of the segment

sigma_seg_B = (cos(beta1)*SIGMA1-cos(beta2)*SIGMA2)/2*h/(h^2+a^2)+...
    (sin(beta2)*SIGMA_p2-sin(beta1)*SIGMA_p1)/2*h/(h^2+a^2); 