function s=stress_seg_iso(MU, NU, b, r1, r2, r, a)
%compute the segmental stress using analytical formulae
%isotropic medium
%solution is not unique

% s(3,3)=calsegstr3d2a(MU, NU, b(3), r1(3), r2(3), r(3), a)
% calls function calsegstrhor3da

r21=r2-r1;
rt=r-r1;

%make sure r21, rt, b are column vector
s1=size(r21); if(s1(1)==1) r21=r21'; end
s2=size(rt); if(s2(1)==1) rt=rt'; end
s3=size(b); if(s3(1)==1) b=b'; end

norm21=norm(r21);
if(norm(r21)<1e-20)
  s=zeros(3,3);
  return;
end

e1=r21/norm21;
e2=schmidt(rt,e1);

e3=cross(e1,e2);

e3=e3/norm(e3);

M=[e2,e3,e1];

Mt=M';

rr21=Mt*r21;
rr=Mt*rt;
br=Mt*b;

%br,rr
sr=calsegstrhor3da(MU,NU,br(1),br(2),br(3),0,rr21(3),rr(1),rr(3),a);
%sr=calsegstrhor3daold(MU,NU,br(1),br(2),br(3),0,rr21(3),rr(1),rr(3),a);

s=M*sr*Mt;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function ss=calsegstrhor3da(MU,NU,bx,by,bz,z1,z2,x,z,a)
%function ss(3,3)=calsegstrhor3da(MU,NU,bx,by,bz,z1,z2,x,z,a)
%based on Hirth-Lothe Eq.(5-45) p.134 (y==0)
%with self consistent smoothing R->Ra

%[MU,NU,bx,by,bz,z1,z2,x,z]'

l=z1-z;
lp=z2-z;
dx=x;
%r=sqrt(dx*dx+l*l);
%rp=sqrt(dx*dx+lp*lp);
ra=sqrt(dx*dx+l*l+a*a);
rap=sqrt(dx*dx+lp*lp+a*a);

dx2=dx*dx;
a2=a*a;

rap2=rap*rap; rap3=rap2*rap;
ra2=ra*ra;    ra3=ra2*ra;

sunit=MU*(1/4/pi/(1-NU));

s=zeros(1,6);
if((l>0)&(lp>0))
  form=1;
elseif((l<0)&(lp<0))
  form=2;
else
  form=3;
end

%for consistency test
%form=3;
%disp(sprintf('form=%d',form));

switch form
case 1,
%  disp('form 1');
  invral=1/ra/(ra+l);
  invrapl=1/rap/(rap+lp);
  s(1) = by*dx*( invrapl*(1-(dx2+a2)/rap2-(dx2+a2)*invrapl)...
               - invral *(1-(dx2+a2)/ra2 -(dx2+a2)*invral) );
  s(2) = -bx*dx*( invrapl - invral );
  s(3) = by*( (-NU/rap+dx2/rap3+(1-NU)*(a2/2)/rap3)...
             -(-NU/ra +dx2/ra3 +(1-NU)*(a2/2)/ra3 ) );
  s(4) = -by*dx*( invrapl*(1+a2/rap2+a2*invrapl)...
                - invral *(1+a2/ra2 +a2*invral ) );
  s(5) = ( bx*(NU/rap-(1-NU)*(a2/2)/rap3) - bz*dx*(1-NU)*invrapl*(1+(a2/2)/rap2+(a2/2)*invrapl) )...
        -( bx*(NU/ra -(1-NU)*(a2/2)/ra3 ) - bz*dx*(1-NU)*invral *(1+(a2/2)/ra2 +(a2/2)*invral ) );
  s(6) = by*dx*( (-2*NU*invrapl*(1+(a2/2)/rap2+(a2/2)*invrapl)-lp/rap3)...
               - (-2*NU*invral *(1+(a2/2)/ra2 +(a2/2)*invral )-l /ra3 ) );
case 2,
%  disp('form 2');
  invral=1/ra/(ra-l);
  invrapl=1/rap/(rap-lp);
%  [invra2,invral,invrap2,invrapl,bx,by,bz,dx,rp,r]'
  s(1) = -by*dx*( invrapl*(1-(dx2+a2)/rap2-(dx2+a2)*invrapl)...
                - invral *(1-(dx2+a2)/ra2 -(dx2+a2)*invral) );
  s(2) = +bx*dx*( invrapl - invral );
  s(3) = by*( (-NU/rap+dx2/rap3+(1-NU)*(a2/2)/rap3)...
             -(-NU/ra +dx2/ra3 +(1-NU)*(a2/2)/ra3 ) );
  s(4) = by*dx*( invrapl*(1+a2/rap2+a2*invrapl)...
                -invral *(1+a2/ra2 +a2*invral ) );
  s(5) = ( bx*(NU/rap-(1-NU)*(a2/2)/rap3) + bz*dx*(1-NU)*invrapl*(1+(a2/2)/rap2+(a2/2)*invrapl) )...
        -( bx*(NU/ra -(1-NU)*(a2/2)/ra3 ) + bz*dx*(1-NU)*invral *(1+(a2/2)/ra2 +(a2/2)*invral ) );
  s(6) = by*dx*( (2*NU*invrapl*(1+(a2/2)/rap2+(a2/2)*invrapl)-lp/rap3)...
               - (2*NU*invral *(1+(a2/2)/ra2 +(a2/2)*invral )-l /ra3 ) );
case 3,
%  disp('form 3');
  rhoa2=dx*dx+a*a;
  s(1) = by*dx/rhoa2 *( lp/rap*(-1+2*(dx2+a2)/rhoa2+(dx2+a2)/rap/rap) ...
                       -l /ra *(-1+2*(dx2+a2)/rhoa2+(dx2+a2)/ra /ra ) );
  s(2) = +bx*dx/rhoa2*( lp/rap - l/ra );
  s(3) = by*( (-NU/rap+dx2/rap3+(1-NU)*(a2/2)/rap3)...
             -(-NU/ra +dx2/ra3 +(1-NU)*(a2/2)/ra3 ) );  
  s(4) = by*dx/rhoa2*( lp/rap*(1+(a2*2)/rhoa2+(a2)/rap/rap)...
                     - l /ra *(1+(a2*2)/rhoa2+(a2)/ra /ra ) );
  s(5) = ( bx*(NU/rap -(1-NU)*(a2/2)/rap3) + bz*dx/rhoa2*(1-NU)*lp/rap*(1+(a2)/rhoa2+(a2/2)/rap2) )...
        -( bx*(NU/ra  -(1-NU)*(a2/2)/ra3 ) + bz*dx/rhoa2*(1-NU)*l /ra *(1+(a2)/rhoa2+(a2/2)/ra2 ) );
  s(6) = by*dx*( (2*NU*lp/rhoa2/rap*(1+(a2)/rhoa2+(a2/2)/rap2) - lp/rap3)...
               - (2*NU*l /rhoa2/ra *(1+(a2)/rhoa2+(a2/2)/ra2 ) - l /ra3 ) );
otherwise,
  disp('invalid choice of integration formula!');
end

ss=sunit*[s(1),s(2),s(3);s(2),s(4),s(5);s(3),s(5),s(6)];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
function e2a=schmidt(e2,e1)
%e2a=schmidt(e2,e1)
% Schmidt ortho-normalize e2 against e1

e3=cross(e1,e2);l3=norm(e3);
if(l3==0)
  e2a=e2;e2a(1)=e2a(1)+1;e3=cross(e1,e2a);l3=norm(e3);
  if(l3==0)
    e2a=e2;e2a(2)=e2a(2)+1;e3=cross(e1,e2a);l3=norm(e3);
  end
end
e3=e3/l3;

e2a=cross(e3,e1);
e2a=e2a/norm(e2a);
    

