function [Q,B,S]=stroh_i(t, C, Nint)
%compute Q B S matrix using integral formalism
%
% t: (1x3 unit row vector) direction of infinite dislocation line
% C: (3x3x3x3, rank-4 tensor) elastic constant tensor
%Nint: number of integration points

% default Nint
if(~exist('Nint'))
    Nint = 21;
end

% Construct Datum coordinate (M,N,t);
[M,N] = rhcoord(t);

%integral step width
domiga=2*pi/Nint;

% initializing integrand
Q_i=zeros(9,Nint);
S_i=zeros(9,Nint);
B_i=zeros(9,Nint);

z3 = zeros(3);
for s=1:Nint    
    %initializing nn nm mn mm and nn_1,2,3
    %each m and n are angle dependent
    nn=z3; nm=z3; mn=z3; mm=z3;
    nn_1=z3; nn_2=z3; nn_3=z3;
    
    omiga = (s-1)*domiga;
    m = M*cos(omiga)+N*sin(omiga);
    n = -M*sin(omiga)+N*cos(omiga);
    
    %defining nn mm nm mn for each theta
     for j=1:3
         for k=1:3
             for i=1:3
                 for l=1:3
                 nn(j,k)=nn(j,k)+n(i)*C(i,j,k,l)*n(l);
                 nm(j,k)=nm(j,k)+n(i)*C(i,j,k,l)*m(l);
                 mm(j,k)=mm(j,k)+m(i)*C(i,j,k,l)*m(l);
                 end
             end
         end
     end
     mn = nm';
          
     %define the inverse of nn
     nn_1=inv(nn);
     %define intermediate matrices
     nn_2=nn_1*nm;
     nn_3=mm-mn*nn_1*nm;
     
     %define Q_i, S_i and B_i
     Q_i(:,s) = reshape(nn_1',9,1);
     S_i(:,s) = reshape(nn_2',9,1);
     B_i(:,s) = reshape(nn_3',9,1);
end

% Q, B, S matrices
Q = -sum(Q_i,2)/Nint; Q = reshape(Q,3,3)';
S = -sum(S_i,2)/Nint; S = reshape(S,3,3)';
B =  sum(B_i,2)/Nint; B = reshape(B,3,3)';
