function [Q,B,S]=stroh_m(t, C)
% calculate Q, B and S  using Stroh matrix Formalism
% t: (1x3) unit row vector, indicating dislocation orientation
% C: (3x3x3x3, rank-4 tensor) elastic constant tensor

%Create orthogonal vectors m0 and n0
[m0,n0] = rhcoord(t);

%Define (mn) matrices 
z3 = zeros(3);nn=z3; nm=z3; mn=z3; mm=z3;
for j=1:3
     for k=1:3
         for i=1:3
             for l=1:3
                 nn(j,k)=nn(j,k)+n0(i)*C(i,j,k,l)*n0(l);
                 nm(j,k)=nm(j,k)+n0(i)*C(i,j,k,l)*m0(l);
                 mm(j,k)=mm(j,k)+m0(i)*C(i,j,k,l)*m0(l);
             end
         end
     end
end
mn = nm';
 
%Create 6x6 matrix N0
N0=zeros(6);
invnn = inv(nn);
N0=-[invnn*nm,invnn;mn*invnn*nm-mm,mn*invnn];
 
%Solve Eigen equation
[V,D]=eig(N0);
 
%initializing A, L and P
A = z3; L = z3; P=zeros(1,3);

%Define A, L and P
for alpha=1:3
    if imag(D(2*alpha,2*alpha))>0
        P(alpha)=D(2*alpha,2*alpha);
        A(:,alpha)=V(1:3,2*alpha);
        L(:,alpha)=V(4:6,2*alpha);
    else
        P(alpha)=D(2*alpha-1,2*alpha-1);
        A(:,alpha)=V(1:3,2*alpha-1);
        L(:,alpha)=V(4:6,2*alpha-1);
    end
end
 
%normalize A and L
for alpha=1:3
    %AialphaLibeta+AibetaLialpha=delta(alpha,beta)
    ralpha = sqrt(2*sum(A(:,alpha).*L(:,alpha)));
    A(:,alpha) = A(:,alpha)/ralpha;
    L(:,alpha) = L(:,alpha)/ralpha;    
end

%Construct Q, B, S matrices using A and L
I=sqrt(-1);
Q = real(2*I*A*A.');
B = real(2*I*L*L.');
S = real(2*I*A*L.'-I*eye(3));