function [Q_prime,B_prime,S_prime] = stroh_p_i(t,n,b,C,N_int)
%calculate the derivative of angular stress factor, SIGMA, wrt theta
%Integral Formalism
% t: (1x3) unit row vector, indicating dislocation orientation
% n: (1x3) unit row vector perpendicular to t, used to define theta
% C: (3x3x3x3, rank-4 tensor) elastic constant tensor
%Nint: number of integration points, for integral formalism only

if(~exist('Nint'))
    N_int = 21; % default number of integration points
end

t = t/norm(t); N = n/norm(n); M=cross(N,t);

[Q,B,S]=stroh_i(t, C, N_int);

domiga=2*pi/N_int;
omigas=0:domiga:2*pi;

z9 = zeros(9,N_int+1);
B_p= z9; S_p = z9; Q_p = z9;

z3 = zeros(3);

for i=1:N_int+1
    omiga=omigas(i);
    %define m and n
    m=M*cos(omiga)+N*sin(omiga);
    n=-M*sin(omiga)+N*cos(omiga);
    %define nn mn nm nt tn mt tm matrices
    %initializing
    nn=z3; mm=z3; mn=z3;
    nm=z3; nt=z3; tn=z3; mt=z3; tm=z3;
    %define ( )'s
    for p=1:3
        for j=1:3
            for k=1:3
                for l=1:3
                    nn(j,k)=nn(j,k)+n(p)*C(p,j,k,l)*n(l);
                    mm(j,k)=mm(j,k)+m(p)*C(p,j,k,l)*m(l);
                    mn(j,k)=mn(j,k)+m(p)*C(p,j,k,l)*n(l);
                    nm(j,k)=nm(j,k)+n(p)*C(p,j,k,l)*m(l);
                    nt(j,k)=nt(j,k)+n(p)*C(p,j,k,l)*t(l);
                    tn(j,k)=tn(j,k)+t(p)*C(p,j,k,l)*n(l);
                    mt(j,k)=mt(j,k)+m(p)*C(p,j,k,l)*t(l);
                    tm(j,k)=tm(j,k)+t(p)*C(p,j,k,l)*m(l);
                end
            end
        end
    end
    
    %define nn_1    
    nn_1=inv(nn);
    
    %define F
    F=nn_1*(nt+tn)*nn_1;
    %define B_i S_i and Q_i
    B_i=-(mt+tm)*cos(omiga)+mn*F*nm*sin(omiga)+(tn*cos(omiga)-...
        mt*sin(omiga))*nn_1*nm-mn*nn_1*(tm*sin(omiga)-nt*cos(omiga));
    S_i=-F*nm*sin(omiga)+nn_1*(tm*sin(omiga)-nt*cos(omiga));
    Q_i=-F*sin(omiga);
    for j=1:3
        for k=1:3
            B_p(3*(j-1)+k,i)=B_i(j,k);
            S_p(3*(j-1)+k,i)=S_i(j,k);
            Q_p(3*(j-1)+k,i)=Q_i(j,k);
        end
    end
end

%calculate B Q S primes
B_prime=z3; S_prime=z3; Q_prime=z3;
for j=1:3
    for k=1:3
        B_prime(j,k)=trapz(omigas,B_p(3*(j-1)+k,:))/2/pi;
        S_prime(j,k)=-trapz(omigas,S_p(3*(j-1)+k,:))/2/pi;
        Q_prime(j,k)=-trapz(omigas,Q_p(3*(j-1)+k,:))/2/pi;
    end
end