function [Q_p,B_p,S_p] = stroh_p_m(t,n,b,C)
%calculate the derivative of angular stress factor, SIGMA, wrt theta
%Matrix Formalism
%
% t: (1x3) unit row vector, indicating dislocation orientation
% n: (1x3) unit row vector perpendicular to t, used to define theta
% C: (3x3x3x3, rank-4 tensor) elastic constant tensor

%m-n-t, RH coordinate system
t=t/norm(t); n=n/norm(n);
m=cross(n,t);

format long

%initializing
z3 = zeros(3);
nn=z3; mm=z3; tt=z3; nt= z3;
tn=z3; mt=z3; tm=z3; mn=z3; nm=z3;
%
%defining (ab)'s
for i=1:3
    for j=1:3
        for k=1:3
            for l=1:3
                nn(j,k)=nn(j,k)+n(i)*C(i,j,k,l)*n(l);
                mm(j,k)=mm(j,k)+m(i)*C(i,j,k,l)*m(l);
                tt(j,k)=tt(j,k)+t(i)*C(i,j,k,l)*t(l);
                nt(j,k)=nt(j,k)+n(i)*C(i,j,k,l)*t(l);
                tn(j,k)=tn(j,k)+t(i)*C(i,j,k,l)*n(l);
                mt(j,k)=mt(j,k)+m(i)*C(i,j,k,l)*t(l);
                tm(j,k)=tm(j,k)+t(i)*C(i,j,k,l)*m(l);
                mn(j,k)=mn(j,k)+m(i)*C(i,j,k,l)*n(l);
                nm(j,k)=nm(j,k)+n(i)*C(i,j,k,l)*m(l);
            end
        end
    end
end
nn_1=inv(nn);

%define N
N=-[nn_1*nm,nn_1;mn*nn_1*nm-mm,mn*nn_1];

%%Solve Eigen equations
[V,D]=eig(N);

%defining KS = [A;L]
KS=zeros(6,6);
P=zeros(1,6);
for alpha=1:3
    if imag(D(2*alpha,2*alpha))>0
        KS(:,alpha)=V(:,2*alpha);
        KS(:,alpha+3)=V(:,2*alpha-1);
        P(alpha)=D(2*alpha,2*alpha);
        P(alpha+3)=D(2*alpha-1,2*alpha-1);        
    else
        KS(:,alpha)=V(:,2*alpha-1);
        KS(:,alpha+3)=V(:,2*alpha);
        P(alpha)=D(2*alpha-1,2*alpha-1);
        P(alpha+3)=D(2*alpha,2*alpha);
    end
end

%normalizing KS
for alpha=1:6
    rt=sqrt(2*KS(1:3,alpha).'*KS(4:6,alpha));
    KS(:,alpha)=KS(:,alpha)/rt;
end

%Intermediate term p3
p3=tn*nn_1*nm+mn*nn_1*nt-tm-mt;

%first derivative of N wrt theta
Np=[nn_1*nt,zeros(3);p3,tn*nn_1];

%define T
T=[zeros(3),eye(3);eye(3),zeros(3)];

%define N1
N1=KS.'*T*Np*KS;

%define KS_p
KS_p=zeros(6);
for beta=1:6
    for alpha=1:6
        if alpha==beta
            KS_p(:,beta)=KS_p(:,beta);
        else
            KS_p(:,beta)=KS_p(:,beta)+N1(alpha,beta)*KS(:,alpha)/(P(beta)-P(alpha));
        end
    end
end

%Calculate Q_p,B_p,S_p
B_p=z3; S_p = z3; Q_p = z3;
for i=1:3
    for j=1:3
        for alpha=1:3
            B_p(i,j)=B_p(i,j)+2*(KS_p(3+i,alpha)*KS(3+j,alpha)+KS(i+3,alpha)*KS_p(j+3,alpha));
            S_p(i,j)=S_p(i,j)+2*(KS_p(i,alpha)*KS(3+j,alpha)+KS(i,alpha)*KS_p(j+3,alpha));
            Q_p(i,j)=Q_p(i,j)+2*(KS_p(i,alpha)*KS(j,alpha)+KS(i,alpha)*KS_p(j,alpha));
        end
    end
end
B_p=-imag(B_p);S_p=-imag(S_p);Q_p=-imag(Q_p);