%Test case of a Frank Read Source activation process
%close dislocation loop or single dislocation arm

cd ..

global Bscrew Bedge Beclimb Bline

%======================================
% Part 1 Basic Parameters

%segment length (unit: angstrom)
L = 1000;
%cut off radius
a = 1;
%applied stress magnitude
dsig = 0.0249;
%total simulation steps
totalsteps = 100;
%dislocation characteristic
charact = 's';
%maximum time step size
%dt0=1e7; 
dt0 = 1e2;
%whether it is a close dislocation loop, "1" for Yes
loop = 1;
%whether the initial dislocation is perturbed (little bowed) "1" = Yes
perturb_config = 1;
%

%========================================
%Part 2 Initial dislocation configuration setup

%aspect ratio for the loop = height/segment length
ha = 4;

%number of free segments
N_seg = 10;
%
d_seg = L/N_seg;
%
if loop ==1
    y = [-L/2, L/2, L/2, -L/2:d_seg:(L/2-d_seg) ];
    z = [0 0 -ha*L/2 -ha*L/2 zeros(1,length(y)-4)];
    flag = [7 7 7 7 zeros(1,length(y)-4)];%7=fixed;0=free
    links = zeros(length(y),8);
    links(:,1) = [1, 5:length(y),2,3,4]';
    links(:,2) = [5:length(y),2,3,4,1]';
    printnode=N_seg/2+4;
    %printnode = 1;
else
    y = [L/2,-L/2:d_seg:(L/2-d_seg)];
    z = zeros(1,length(y));
    flag = [7,7,zeros(1,length(y)-2)];
    links = zeros(length(y)-1,8);
    links(:,1) = [2:length(y)]';
    links(:,2) = [3:length(y),1]';
    printnode = 2 + N_seg/2;
end
%
if perturb_config == 1
    A = L/50;
    x = A*cos(y*pi/L);
else
    x = zeros(1, length(y));
end
%    
rn = [x;y;z;flag]';
%
if charact == 'e'
    %Edge dislocation segment
    links(:,3) = 1;
    links(:,8) = 1;
    appstr0 = [0  0  -1; 0 0 0; -1 0 0];
    %
elseif charact == 's'
%Screw Dislocation segment
    links(:,4) = 1;
    links(:,8) = 1;
    appstr0 = [0  0  0; 0 0 -1; 0 -1 0];
    %
elseif charact == 'm'
%mixed Dislocation
    links(:,3) = 1;
    links(:,4) = 1;
    links(:,8) = 1;
    appstr0 = [0 0 -1; 0 0 -1; -1 -1 0];
    %
end

%====================================
%Part 3 Simulation Parameters

%Mo
%c44=11.0;
%c11=46.0;
%c12=17.6;

%Fe
%c44=11.2;
%c11=24.2;
%c12=14.65;

%W
c44=16.0001;
c11=52.1;
c12=20.1;
%
%tungsten
MU = 16.0;
NU = 0.278;
%

C = cubicelasticconst(c11,c12,c44);

%Topological constraints
maxconnections=8;
lmax = L;
lmin = L/50;
areamin=lmin*lmin*sin(60/180*pi)*0.5; 
areamax=20*areamin;

%Core energy
%Ec = MU/(4*pi)*log(a/0.1);
Ec = 0;

%Mobility law (for most cases)
mobility='mobfcc1';

%Drag (1/Mobility) parameters
Bscrew=1e0;
Bedge=1e0;
Beclimb=1e2;
Bline=1.0e-4*min(Bscrew,Bedge);

integrator='int_trapezoid';
rann = 0.001;       
rntol = 0.01*L;
%rann = 0.5*a;       
%rntol = 0.5*rann;  

%no remesh enabled
doremesh=0;
docollision=0;
doseparation=0;
elasticinteraction=1; %line tension model when this is zero
plotfreq=1;       
plim=5*max(max(rn));

viewangle=[0 90];
printfreq=1;      

rmax=100;
appliedstress = dsig*appstr0;

disp('now you need to type:');
disp(' cd ..; totalsteps = 10; plotfreq=10; tic; dd3d; t=toc/totalsteps; cd tests');
disp(' ');
disp('if you want to compare the time for iso code, do');
disp(' cd ../src; totalsteps = 1000; plotfreq=100; printfreq=100; mobility=''mobfcc0''; tic; dd3d; t=toc/totalsteps');

%tic
%dd3d
%t = toc

cd tests
%==========================end==========================================
