%compute the stress field of infinite long dislocation segment
%in isotropic media (W)
%
%data included in Table 1 from the following paper in MSMSE
%Efficient Computation of Forces on Dislocation Segments in Anisotropic Elasticity
%Yin J., Barnett, D.M., Cai, W.
%
%============================================================

cd ..

%Parameter for isotropic W (1e10 Pa, from Hirth-Lothe "Theory of Dislocations", p.837)
c11=52.1;c12=20.1;c44=16.0;
mu=c44;lamda=c12;nu=c12/(c11+c12);

%Dislocation information
t=[1 3 4];%line direction
x=[3 5 -7];%field point
b=[1 -2 1]/2;%Burgers vector

% Analytical result (reference)
sigma_ref=stress_inf_iso(x,t,b,mu,nu);

%================================================================

%Part 1 Matrix Formalism

%Brown's formula is the default
C = cubicelasticconst(c11,c12,c44);
Niter=1000;%compute Niter times
meth = 'm';

%using Brown's formula
tic
for iter=1:Niter
    sigma_m = stress_inf_B(x, t, b, C);
end
t_m = toc/Niter;

%using WSL formula
% tic
% for iter=1:Niter
%     sigma_m = stress_inf_WS(x, t, b, C, meth);
% end
% t_m = toc/Niter;

%===================================================

%Part 2 Integral Formalism

meth = 'i';
%unperturbed C
C = cubicelasticconst(c11,c12,c44,'i');

%Nint = 5
Nint = 5;

tic
for iter=1:Niter
    sigma_B_5 = stress_inf_B(x, t, b, C, meth,Nint);
end
t_B_5 = toc/Niter;

%Nint = 11
Nint = 11;

tic
for iter=1:Niter
    sigma_B_11 = stress_inf_B(x, t, b, C, meth,Nint);
end
t_B_11 = toc/Niter;

%N = 21
Nint = 21;

tic
for iter=1:Niter
    sigma_B_21 = stress_inf_B(x, t, b, C, meth,Nint);
end
t_B_21 = toc/Niter;

%=====================================================
%Part 3 Calculate Relative Errors

RE_m = Error_m(sigma_m, sigma_ref);
RE_B_5 = Error_m(sigma_B_5, sigma_ref);
RE_B_11 = Error_m(sigma_B_11, sigma_ref);
RE_B_21 = Error_m(sigma_B_21, sigma_ref);


%Print out results

disp(sprintf('Using M/B method,           t = %e  Relative Error = %e',t_m,RE_m));
disp(sprintf('Using I/B method, N = 5,   t = %e  Relative Error = %e',t_B_5,RE_B_5));
disp(sprintf('Using I/B method, N = 11,  t = %e  Relative Error = %e',t_B_11,RE_B_11));
disp(sprintf('Using I/B method, N = 21,  t = %e  Relative Error = %e',t_B_21,RE_B_21));

cd tests
%==========================end==========================================
