%compute the stress field of a quadrangle dislocation loop
%in anisotropic media (Mo)
%
%data included in Table 3 from the following paper in MSMSE
%Efficient Computation of Forces on Dislocation Segments in Anisotropic Elasticity
%Yin J., Barnett, D.M., Cai, W.
%
%============================================================

cd ..

%Parameter for isotropic Mo (1e10 Pa, from Hirth-Lothe "Theory of Dislocations", p.837)
c44=11.0;
c11=46.0;
c12=17.6;

%Dislocation information
%loop setup ABDE
A = [1 3 3];B = [3 1 5];D = [5 3 2];E = [3 7 4];
%field point
P = [3 2 6];
%Burgers vector
b = [1 2 -1];

%computation reference
%Browns formula + Integral formalism
%Nint = 1e4

%zero cut off radius
a = 0;

C = cubicelasticconst(c11,c12,c44);

Nint = 1e4;
s1_ref=stress_seg_B(A,B,P,b,C,'i',a,Nint);
s2_ref=stress_seg_B(B,D,P,b,C,'i',a,Nint);
s3_ref=stress_seg_B(D,E,P,b,C,'i',a,Nint);
s4_ref=stress_seg_B(E,A,P,b,C,'i',a,Nint);
sigma_ref = s1_ref+s2_ref+s3_ref+s4_ref;

%=======================================

%Part 1 Brown's Formula

%1.1 Using Matrix formalism

Niter = 100;

tic
for iter=1:Niter
    s1_mB=stress_seg_B(A,B,P,b,C,'m',a);
    s2_mB=stress_seg_B(B,D,P,b,C,'m',a);
    s3_mB=stress_seg_B(D,E,P,b,C,'m',a);
    s4_mB=stress_seg_B(E,A,P,b,C,'m',a);
    sigma_mB = s1_mB+s2_mB+s3_mB+s4_mB;
end
t_mB = toc/Niter;

%1.2 Using integral formalism

%Nint = 5
Nint = 5;
tic
for iter=1:Niter
    s1_iB5=stress_seg_B(A,B,P,b,C,'i',a,Nint);
    s2_iB5=stress_seg_B(B,D,P,b,C,'i',a,Nint);
    s3_iB5=stress_seg_B(D,E,P,b,C,'i',a,Nint);
    s4_iB5=stress_seg_B(E,A,P,b,C,'i',a,Nint);
    sigma_iB5 = s1_iB5+s2_iB5+s3_iB5+s4_iB5;
end
t_iB5 = toc/Niter;

%Nint = 11
Nint = 11;
tic
for iter=1:Niter
    s1_iB11=stress_seg_B(A,B,P,b,C,'i',a,Nint);
    s2_iB11=stress_seg_B(B,D,P,b,C,'i',a,Nint);
    s3_iB11=stress_seg_B(D,E,P,b,C,'i',a,Nint);
    s4_iB11=stress_seg_B(E,A,P,b,C,'i',a,Nint);
    sigma_iB11 = s1_iB11+s2_iB11+s3_iB11+s4_iB11;
end
t_iB11 = toc/Niter;

%Nint = 21
Nint = 21;
tic
for iter=1:Niter
    s1_iB21=stress_seg_B(A,B,P,b,C,'i',a,Nint);
    s2_iB21=stress_seg_B(B,D,P,b,C,'i',a,Nint);
    s3_iB21=stress_seg_B(D,E,P,b,C,'i',a,Nint);
    s4_iB21=stress_seg_B(E,A,P,b,C,'i',a,Nint);
    sigma_iB21 = s1_iB21+s2_iB21+s3_iB21+s4_iB21;
end
t_iB21 = toc/Niter;

%========================================
%Part 2 Willis-Steeds-Lothe (WSL) Formula

%2.1 Using Matrix formalism

Niter = 1;

tic
for iter=1:Niter
    s1_mWSL=stress_seg_WS(A,B,P,b,C,'m',a);
    s2_mWSL=stress_seg_WS(B,D,P,b,C,'m',a);
    s3_mWSL=stress_seg_WS(D,E,P,b,C,'m',a);
    s4_mWSL=stress_seg_WS(E,A,P,b,C,'m',a);
    sigma_mWSL = s1_mWSL+s2_mWSL+s3_mWSL+s4_mWSL;
end
t_mWSL = toc/Niter;

%2.2 Using integral formalism

%Nint = 5
Nint = 5;
tic
for iter=1:Niter
    s1_iWSL5=stress_seg_WS(A,B,P,b,C,'i',a,Nint);
    s2_iWSL5=stress_seg_WS(B,D,P,b,C,'i',a,Nint);
    s3_iWSL5=stress_seg_WS(D,E,P,b,C,'i',a,Nint);
    s4_iWSL5=stress_seg_WS(E,A,P,b,C,'i',a,Nint);
    sigma_iWSL5 = s1_iWSL5+s2_iWSL5+s3_iWSL5+s4_iWSL5;
end
t_iWSL5 = toc/Niter;

%Nint = 11
Nint = 11;
tic
for iter=1:Niter
    s1_iWSL11=stress_seg_WS(A,B,P,b,C,'i',a,Nint);
    s2_iWSL11=stress_seg_WS(B,D,P,b,C,'i',a,Nint);
    s3_iWSL11=stress_seg_WS(D,E,P,b,C,'i',a,Nint);
    s4_iWSL11=stress_seg_WS(E,A,P,b,C,'i',a,Nint);
    sigma_iWSL11 = s1_iWSL11+s2_iWSL11+s3_iWSL11+s4_iWSL11;
end
t_iWSL11 = toc/Niter;

%Nint = 21
Nint = 21;
tic
for iter=1:Niter
    s1_iWSL21=stress_seg_WS(A,B,P,b,C,'i',a,Nint);
    s2_iWSL21=stress_seg_WS(B,D,P,b,C,'i',a,Nint);
    s3_iWSL21=stress_seg_WS(D,E,P,b,C,'i',a,Nint);
    s4_iWSL21=stress_seg_WS(E,A,P,b,C,'i',a,Nint);
    sigma_iWSL21 = s1_iWSL21+s2_iWSL21+s3_iWSL21+s4_iWSL21;
end
t_iWSL21 = toc/Niter;

%===================================================
%Part 3 Compute relative errors

RE_mB = Error_m(sigma_mB, sigma_ref);
RE_iB5 = Error_m(sigma_iB5, sigma_ref);
RE_iB11 = Error_m(sigma_iB11, sigma_ref);
RE_iB21 = Error_m(sigma_iB21, sigma_ref);

RE_mWSL = Error_m(sigma_mWSL, sigma_ref);
RE_iWSL5 = Error_m(sigma_iWSL5, sigma_ref);
RE_iWSL11 = Error_m(sigma_iWSL11, sigma_ref);
RE_iWSL21 = Error_m(sigma_iWSL21, sigma_ref);

%=================================================================
%Part 4 Print results

%Print out results for Part 1
%
disp(sprintf('Using M/B method,           t = %e  Relative Error = %e',t_mB,RE_mB));
disp(sprintf('Using I/B method, N = 5,   t = %e  Relative Error = %e',t_iB5,RE_iB5));
disp(sprintf('Using I/B method, N = 11,  t = %e  Relative Error = %e',t_iB11,RE_iB11));
disp(sprintf('Using I/B method, N = 21,  t = %e  Relative Error = %e',t_iB21,RE_iB21));
%
%Pring out results for Part 2
disp(sprintf('Using M/WSL method,           t = %e  Relative Error = %e',t_mWSL,RE_mWSL));
disp(sprintf('Using I/WSL method, N = 5,   t = %e  Relative Error = %e',t_iWSL5,RE_iWSL5));
disp(sprintf('Using I/WSL method, N = 11,  t = %e  Relative Error = %e',t_iWSL11,RE_iWSL11));
disp(sprintf('Using I/WSL method, N = 21,  t = %e  Relative Error = %e',t_iWSL21,RE_iWSL21));

cd tests
%==========================end==========================================